import graphics
import random
import pyglet
import Box2D as box2d
import math
import physics
import settings

def make_triangle(size):
    rad = 0
    rad2 = graphics.radian(120)
    rad3 = graphics.radian(240)
    ax, ay = size*math.cos(rad), size*math.sin(rad)
    bx, by = size*math.cos(rad2), size*math.sin(rad2)
    cx, cy = size*math.cos(rad3), size*math.sin(rad3)
    return [(ax, ay), (bx, by), (cx, cy)]

class Murder(object):
    def __init__(self, world):
        self.world_context = world
        if settings.crow_shape == 'triangle':
            self.crow_shape_def = physics.PolygonDef(world.world, make_triangle(settings.crow_size))
        else:
            self.crow_shape_def = physics.CircleDef(world.world, settings.crow_size)
        self.crows = []
    
    def add_crow(self, target):
        crow = Crow(self, abs(random.random()) *settings.world_width, abs(random.random())*(settings.world_height/2) + settings.world_height/2)
        crow.target = target
        self.crows.append(crow)
    
    def update(self, dt):
        for crow in self.crows:
            crow.update(dt)
    
    def draw(self):
        gl = pyglet.gl.gl
        gl.glColor4f(0, 0, 0, 1)
        batch = pyglet.graphics.Batch()
        for crow in self.crows:
            crow.draw(batch)
        batch.draw()
        pyglet.gl.gl.glColor4f(0, 0, 0, 1)

class Crow(object):
    def __init__(self, murder, x, y):
        self.size = settings.crow_size
        self.anim = 0
        self.murder = murder
        self.target = None
        self.phys_object = physics.Polygon(murder.crow_shape_def, (x,y) )
        self.phys_object.body.userData = self
        self.life = 50
        self.speed = settings.crow_idle_speed
        self.blood_lust = 0
        self.resting = False
    
    def on_hit(self, contact):
        pass
    
    def on_remove_hit(self, contact):
        pass
    
    def damage(self, contact):
        from biped import Biped
        if isinstance(contact.shape1.GetBody().userData, Biped) or isinstance(contact.shape2.GetBody().userData, Biped):
            self.blood_lust = 1.5
            self.resting = True
            self.target = box2d.b2Vec2(settings.world_width*random.random(), settings.world_height-(random.random()*settings.world_height*0.3))
            self.speed = settings.crow_idle_speed
    
    def moveTowards(self, x, y):
        if not self.resting: 
            self.target = box2d.b2Vec2(x, y)
    
    def update(self, dt):
        #limit speed
        self.blood_lust -= dt
        if self.blood_lust > 0:
            self.phys_object.body.ApplyTorque(self.blood_lust*80)
        else:
            self.blood_lust = 0
            self.resting = False
        velocity = self.phys_object.body.GetLinearVelocity()
        if velocity.Length() > self.speed:
            velocity.Normalize()
            velocity = velocity*self.speed
            self.phys_object.body.SetLinearVelocity(velocity)
        self.anim += dt
        if self.target:
            nose = self.phys_object.body.GetWorldPoint((0, -self.size))
            
            #self.phys_object.body.SetLinearVelocity( (self.target - self.phys_object.body.position)*40)
            self.phys_object.body.ApplyForce( (self.target - self.phys_object.body.position)*settings.crow_speed, nose)
            #print self.phys_object.body.GetLinearVelocity().Length()
    
    def draw(self, batch):
        pyglet.gl.gl.glColor4f(self.blood_lust*0.15, 0, 0, 1-self.blood_lust*0.25)
        self.phys_object.draw()
        
